@@ -1,4 +1,4 @@
-process_layers <- function(g, z, gt, gf, allow.small.mult) {
+process_layers <- function(g, z, gt, gf, interactive) {
 	if (dupl <- anyDuplicated(names(g))) {
 		warning("One tm layer group has duplicated layer types, which are omitted. To draw multiple layers of the same type, use multiple layer groups (i.e. specify tm_shape prior to each of them).", call. = FALSE)
 		g <- g[-dupl]	
@@ -13,8 +13,8 @@ process_layers <- function(g, z, gt, gf, allow.small.mult) {
 		stop(g$tm_shape$shp_name, " is a raster, so it cannot accept tm_fill/tm_borders/tm_polygons.", call. = FALSE)
 	} else if (type=="raster" && "tm_lines" %in% names(g)) {
 		stop(g$tm_shape$shp_name, " is a raster, so it cannot accept tm_lines.", call. = FALSE)
-	} else if (type=="raster" && "tm_bubbles" %in% names(g)) {
-		stop(g$tm_shape$shp_name, " is a raster, so it cannot accept tm_bubbles/tm_dots.", call. = FALSE)
+	} else if (type=="raster" && "tm_symbols" %in% names(g)) {
+		stop(g$tm_shape$shp_name, " is a raster, so it cannot accept tm_symbols/tm_dots.", call. = FALSE)
 	} else if (type=="points" && "tm_lines" %in% names(g)) {
 		stop(g$tm_shape$shp_name, " consists of spatial points, so it cannot accept tm_lines.", call. = FALSE)
 	} else if (type=="points" && "tm_raster" %in% names(g)) {
@@ -71,7 +71,7 @@ process_layers <- function(g, z, gt, gf, allow.small.mult) {
 	}
 
 	# determine plotting order 
-	plot.order <- names(g)[names(g) %in% c("tm_fill", "tm_borders", "tm_text", "tm_bubbles", "tm_lines", "tm_raster")]
+	plot.order <- names(g)[names(g) %in% c("tm_fill", "tm_borders", "tm_text", "tm_symbols", "tm_lines", "tm_raster") | substr(names(g), 1, 13)=="tm_add_legend"]
 	plot.order[plot.order=="tm_borders"] <- "tm_fill"
 	plot.order <- unique(plot.order)
 	
@@ -95,27 +95,27 @@ process_layers <- function(g, z, gt, gf, allow.small.mult) {
 	if (is.null(g$tm_fil)) {
 		gfill <- list(fill=NULL, xfill=NA, fill.legend.title=NA, fill.id=NA) 
 	} else {
-		gfill <- process_fill(data, g$tm_fill, gborders, gt, gf, z=z+which(plot.order=="tm_fill"), allow.small.mult=allow.small.mult)
+		gfill <- process_fill(data, g$tm_fill, gborders, gt, gf, z=z+which(plot.order=="tm_fill"), interactive=interactive)
 	}
-	# bubble info
-	if (is.null(g$tm_bubbles)) {
-		gbubble <- list(bubble.size=NULL, xsize=NA, xcol=NA, bubble.size.legend.title=NA, bubble.col.legend.title=NA, bubble.id=NA)
+	# symbol info
+	if (is.null(g$tm_symbols)) {
+		gsymbol <- list(symbol.size=NULL, xsize=NA, xcol=NA, xshape=NA, symbol.size.legend.title=NA, symbol.col.legend.title=NA, symbol.shape.legend.title=NA, symbol.id=NA)
 	} else {
-		gbubble <- process_bubbles(data, g$tm_bubbles, gt, gf, z=z+which(plot.order=="tm_bubbles"), allow.small.mult=allow.small.mult)
+		gsymbol <- process_symbols(data, g$tm_symbols, gt, gf, z=z+which(plot.order=="tm_symbols"), interactive=interactive)
 	}
 
 	# lines info
 	if (is.null(g$tm_lines)) {
 		glines <- list(line.lwd=NULL, xline=NA, xlinelwd=NA, line.col.legend.title=NA, line.lwd.legend.title=NA, line.id=NA) 
 	} else {
-		glines <- process_lines(data, g$tm_lines, gt, gf, z=z+which(plot.order=="tm_lines"), allow.small.mult=allow.small.mult)	
+		glines <- process_lines(data, g$tm_lines, gt, gf, z=z+which(plot.order=="tm_lines"), interactive=interactive)	
 	} 
 
 	# raster info
 	if (is.null(g$tm_raster)) {
 		graster <- list(raster=NULL, xraster=NA, raster.legend.title=NA) 
 	} else {
-		graster <- process_raster(data, g$tm_raster, gt, gf, z=z+which(plot.order=="tm_raster"), allow.small.mult=allow.small.mult)
+		graster <- process_raster(data, g$tm_raster, gt, gf, z=z+which(plot.order=="tm_raster"), interactive=interactive)
 	}	
 	
 	
@@ -123,11 +123,27 @@ process_layers <- function(g, z, gt, gf, allow.small.mult) {
 	if (is.null(g$tm_text)) {
 		gtext <- list(text=NULL, xtext=NA, xtsize=NA, xtcol=NA, text.size.legend.title=NA, text.col.legend.title=NA)
 	}  else {
-		gtext <- process_text(data, g$tm_text, if (is.null(gfill$fill)) NA else gfill$fill, gt, gf, z=z+which(plot.order=="tm_text"), allow.small.mult=allow.small.mult)
+		gtext <- process_text(data, g$tm_text, if (is.null(gfill$fill)) NA else gfill$fill, gt, gf, z=z+which(plot.order=="tm_text"), interactive=interactive)
 	}
 
-	any.legend <- any(!is.ena(c(gfill$fill.legend.title, gbubble$bubble.size.legend.title, gbubble$bubble.col.legend.title, glines$line.col.legend.title, glines$line.lwd.legend.title, graster$raster.legend.title, gtext$text.size.legend.title, gtext$text.col.legend.title)))
+	any.legend <- any(!is.ena(c(gfill$fill.legend.title, gsymbol$symbol.size.legend.title, gsymbol$symbol.col.legend.title, gsymbol$symbol.shape.legend.title, glines$line.col.legend.title, glines$line.lwd.legend.title, graster$raster.legend.title, gtext$text.size.legend.title, gtext$text.col.legend.title)))
 	# 	glines$line.lwd.legend.title
 	
-	c(list(npol=nrow(data), varnames=list(by=by, fill=gfill$xfill, bubble.size=gbubble$xsize, bubble.col=gbubble$xcol, line.col=glines$xline, line.lwd=glines$xlinelwd, raster=graster$xraster, text.size=gtext$xtsize, text.col=gtext$xtcol), idnames=list(fill=gfill$fill.id, bubble=gbubble$bubble.id, line=glines$line.id, raster=graster$raster.id, text=gtext$text.id), data_by=data$GROUP_BY, nrow=nrow, ncol=ncol, panel.names=panel.names, plot.order=plot.order, any.legend=any.legend), gborders, gfill, glines, gbubble, gtext, graster)
+	
+	
+	legids <- which(substr(names(g), 1, 13)=="tm_add_legend")
+	
+	if (length(legids)) {
+		add_legends <- mapply(function(gal, name) {
+			if (is.na(gal$z)) gal$z <- z + which(plot.order==name)
+			gal
+		}, g[legids], names(g[legids]), SIMPLIFY=FALSE)
+		
+	} else {
+		add_legends <- list()
+	}
+	
+	plot.order <- plot.order[substr(plot.order, 1, 13)!="tm_add_legend"]
+	
+	c(list(npol=nrow(data), varnames=list(by=by, fill=gfill$xfill, symbol.size=gsymbol$xsize, symbol.col=gsymbol$xcol, symbol.shape=gsymbol$xshape, line.col=glines$xline, line.lwd=glines$xlinelwd, raster=graster$xraster, text.size=gtext$xtsize, text.col=gtext$xtcol), idnames=list(fill=gfill$fill.id, symbol=gsymbol$symbol.id, line=glines$line.id), data_by=data$GROUP_BY, nrow=nrow, ncol=ncol, panel.names=panel.names, plot.order=plot.order, any.legend=any.legend), gborders, gfill, glines, gsymbol, gtext, graster, list(add_legends=add_legends))
 }
\ No newline at end of file
